@@ -73,8 +73,7 @@ static const GtkActionEntry battstat_menu_actions [] = {
 #define DC_POWER_STRING _("System is running on battery power")
 
 /* The icons for Battery, Critical, AC and Charging */
-static GdkPixmap *statusimage[STATUS_PIXMAP_NUM];
-static GdkBitmap *statusmask[STATUS_PIXMAP_NUM];
+static GdkPixbuf *statusimage[STATUS_PIXMAP_NUM];
 
 /* Assuming a horizontal battery, the colour is drawn into it one horizontal
    line at a time as a vertical gradient.  The following arrays decide where
@@ -218,54 +217,20 @@ static GdkColor darkred[] = {
 static void
 initialise_global_pixmaps( void )
 {
-  GdkDrawable *defaults;
-
-  defaults = gdk_screen_get_root_window( gdk_screen_get_default() );
-
   statusimage[STATUS_PIXMAP_BATTERY] =
-    gdk_pixmap_create_from_xpm_d( defaults, &statusmask[STATUS_PIXMAP_BATTERY],
-                                  NULL, battery_small_xpm );
+    gdk_pixbuf_new_from_xpm_data ((const char **) battery_small_xpm);
 
   statusimage[STATUS_PIXMAP_METER] =
-    gdk_pixmap_create_from_xpm_d( defaults, &statusmask[STATUS_PIXMAP_METER],
-                                  NULL, battery_small_meter_xpm );
+    gdk_pixbuf_new_from_xpm_data ((const char **) battery_small_meter_xpm);
 
   statusimage[STATUS_PIXMAP_AC] =
-    gdk_pixmap_create_from_xpm_d( defaults, &statusmask[STATUS_PIXMAP_AC],
-                                  NULL, ac_small_xpm );
+    gdk_pixbuf_new_from_xpm_data ((const char **) ac_small_xpm);
    
   statusimage[STATUS_PIXMAP_CHARGE] =
-    gdk_pixmap_create_from_xpm_d( defaults, &statusmask[STATUS_PIXMAP_CHARGE],
-                                  NULL, charge_small_xpm );
+    gdk_pixbuf_new_from_xpm_data ((const char **) charge_small_xpm);
    
   statusimage[STATUS_PIXMAP_WARNING] =
-    gdk_pixmap_create_from_xpm_d( defaults, &statusmask[STATUS_PIXMAP_WARNING],
-                                  NULL, warning_small_xpm );
-}
-
-/* For non-truecolour displays, each GdkColor has to have a palette entry
-   allocated for it.  This should only be done once for the entire display.
-*/
-static void
-allocate_battery_colours( void )
-{
-  GdkColormap *colourmap;
-  int i;
-
-  colourmap = gdk_colormap_get_system();
-
-  /* assumed: all the colour arrays have the same number of elements */
-  for( i = 0; i < G_N_ELEMENTS( orange ); i++ )
-  {
-     gdk_colormap_alloc_color( colourmap, &darkorange[i], FALSE, TRUE );
-     gdk_colormap_alloc_color( colourmap, &darkyellow[i], FALSE, TRUE );
-     gdk_colormap_alloc_color( colourmap, &darkred[i], FALSE, TRUE );
-     gdk_colormap_alloc_color( colourmap, &darkgreen[i], FALSE, TRUE );
-     gdk_colormap_alloc_color( colourmap, &orange[i], FALSE, TRUE );
-     gdk_colormap_alloc_color( colourmap, &yellow[i], FALSE, TRUE );
-     gdk_colormap_alloc_color( colourmap, &red[i], FALSE, TRUE );
-     gdk_colormap_alloc_color( colourmap, &green[i], FALSE, TRUE );
-  }
+    gdk_pixbuf_new_from_xpm_data ((const char **) warning_small_xpm);
 }
 
 /* Our backends may be either event driven or poll-based.
@@ -318,7 +283,6 @@ static_global_initialisation (int no_hal, ProgressData *battstat)
   if (!first_time)
     return NULL;
 
-  allocate_battery_colours();
   initialise_global_pixmaps();
   err = power_management_initialise (no_hal, status_change_callback);
 
@@ -352,9 +316,9 @@ battstat_error_dialog( GtkWidget *applet, const char *msg )
   gtk_window_set_screen( GTK_WINDOW (dialog),
                          gtk_widget_get_screen (GTK_WIDGET (applet)) );
 
-  g_signal_connect_swapped( GTK_OBJECT (dialog), "response",
+  g_signal_connect_swapped( G_OBJECT (dialog), "response",
                             G_CALLBACK (gtk_widget_destroy),
-                            GTK_OBJECT (dialog) );
+                            G_OBJECT (dialog) );
 
   gtk_widget_show_all( dialog );
 }
@@ -435,7 +399,7 @@ battery_full_notify (GtkWidget *applet)
 			GTK_ICON_LOOKUP_USE_BUILTIN,
 			NULL);
 	
-	NotifyNotification *n = notify_notification_new (_("Your battery is now fully recharged"), "", /* "battery" */ NULL, applet);
+	NotifyNotification *n = notify_notification_new (_("Your battery is now fully recharged"), "", /* "battery" */ NULL);
 
 	/* XXX: it would be nice to pass this as a named icon */
 	notify_notification_set_icon_from_pixbuf (n, icon);
@@ -477,12 +441,11 @@ battery_full_dialog (GtkWidget *applet)
 		GTK_STOCK_OK,
 		GTK_RESPONSE_ACCEPT,
 		NULL);
-  g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
+  g_signal_connect_swapped (G_OBJECT (dialog), "response",
 			    G_CALLBACK (gtk_widget_destroy),
-			    GTK_OBJECT (dialog));
+			    G_OBJECT (dialog));
 
   gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
-  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
   hbox = gtk_hbox_new (FALSE, 6);
   pixbuf = gtk_icon_theme_load_icon (
 		gtk_icon_theme_get_default (),
@@ -636,15 +599,13 @@ battery_low_dialog( ProgressData *battery, BatteryStatus *info )
   gtk_dialog_set_default_response( GTK_DIALOG (battery->battery_low_dialog),
                                    GTK_RESPONSE_ACCEPT );
 
-  g_signal_connect_swapped( GTK_OBJECT (battery->battery_low_dialog),
+  g_signal_connect_swapped( G_OBJECT (battery->battery_low_dialog),
                             "response",
                             G_CALLBACK (battery_low_dialog_destroy),
                             battery );
 
   gtk_container_set_border_width (GTK_CONTAINER (battery->battery_low_dialog),
 		  6);
-  gtk_dialog_set_has_separator (GTK_DIALOG (battery->battery_low_dialog),
-		  FALSE);
   hbox = gtk_hbox_new (FALSE, 6);
   gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
   pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
@@ -713,14 +674,54 @@ update_tooltip( ProgressData *battstat, BatteryStatus *info )
   g_free (tiptext);
 }
 
+static void
+pixbuf_draw_line( GdkPixbuf *pixbuf, GdkColor *colour, int x1, int y1, int x2, int y2)
+{
+  guchar *pixels = gdk_pixbuf_get_pixels (pixbuf);
+  int stride = gdk_pixbuf_get_rowstride (pixbuf);
+  int channels = gdk_pixbuf_get_n_channels (pixbuf);
+  guchar r, g, b, a;
+  int i, n;
+
+  r = colour->red >> 8;
+  g = colour->green >> 8;
+  b = colour->blue >> 8;
+  a = 0xFF;
+
+  pixels += stride * y1 + 4 * x1;
+
+  if (x1 == x2)
+  {
+    /* stride = gdk_pixbuf_get_rowstride (pixbuf); */
+    n = y2 - y1;
+  }
+  else if (y1 == y2)
+  {
+    stride = gdk_pixbuf_get_n_channels (pixbuf);
+    n = x2 - x1;
+  }
+  else
+    g_assert_not_reached ();
+
+  for (i = 0; i < n; i++)
+  {
+    pixels[0] = r;
+    pixels[1] = g;
+    pixels[2] = b;
+    if (channels == 4)
+      pixels[3] = a;
+
+    pixels += stride;
+  }
+}
+
 /* Redraw the battery meter image.
  */
 static void
 update_battery_image (ProgressData *battstat, int batt_percent, int batt_time)
 {
   GdkColor *color, *darkcolor;
-  GdkPixmap *pixmap;
-  GdkBitmap *pixmask;
+  GdkPixbuf *pixbuf;
   guint progress_value;
   gint i, x;
   int batt_life;
@@ -751,25 +752,13 @@ update_battery_image (ProgressData *battstat, int batt_percent, int batt_time)
     darkcolor = darkgreen;
   }
 
-  /* We keep this pixgc allocated so we don't have to alloc/free it every
-     time.  A widget has to be realized before it has a valid ->window so
-     we do that here for battstat->applet just in case it's not already done.
-  */
-  if( battstat->pixgc == NULL )
-  {
-    gtk_widget_realize( battstat->applet );
-    battstat->pixgc = gdk_gc_new( gtk_widget_get_window (battstat->applet) );
-  }
-
   /* Depending on if the meter is horizontally oriented start out with the
      appropriate XPM image (from pixmaps.h)
   */
   if (battstat->horizont)
-    pixmap = gdk_pixmap_create_from_xpm_d( gtk_widget_get_window (battstat->applet), &pixmask,
-                                           NULL, battery_gray_xpm );
+    pixbuf = gdk_pixbuf_new_from_xpm_data ((const char **) battery_gray_xpm);
   else
-    pixmap = gdk_pixmap_create_from_xpm_d( gtk_widget_get_window (battstat->applet), &pixmask,
-                                           NULL, battery_y_gray_xpm );
+    pixbuf = gdk_pixbuf_new_from_xpm_data ((const char **) battery_y_gray_xpm);
 
   /* The core code responsible for painting the battery meter.  For each
      colour in our gradient array, draw a vertical or horizontal line
@@ -780,13 +769,11 @@ update_battery_image (ProgressData *battstat, int batt_percent, int batt_time)
 	    
     for( i = 0; i < G_N_ELEMENTS( orange ); i++ )
     {
-      gdk_gc_set_foreground (battstat->pixgc, &color[i]);
-
       if (battstat->horizont)
-        gdk_draw_line (pixmap, battstat->pixgc, pixel_offset_top[i], i + 2,
+        pixbuf_draw_line (pixbuf, &color[i], pixel_offset_top[i], i + 2,
                        pixel_offset_top[i] + progress_value, i + 2);
       else
-        gdk_draw_line (pixmap, battstat->pixgc, i + 2, pixel_offset_top[i],
+        pixbuf_draw_line (pixbuf, &color[i], i + 2, pixel_offset_top[i],
                        i + 2, pixel_offset_top[i] + progress_value);
     }
   }
@@ -796,15 +783,13 @@ update_battery_image (ProgressData *battstat, int batt_percent, int batt_time)
 
     for( i = 0; i < G_N_ELEMENTS( orange ); i++)
     {
-      gdk_gc_set_foreground (battstat->pixgc, &color[i]);
-
       if (battstat->horizont)
-        gdk_draw_line (pixmap, battstat->pixgc, pixel_offset_bottom[i], i + 2,
-                       pixel_offset_bottom[i] - progress_value, i + 2);
+        pixbuf_draw_line (pixbuf, &color[i], pixel_offset_bottom[i], i + 2,
+                          pixel_offset_bottom[i] - progress_value, i + 2);
       else
-        gdk_draw_line (pixmap, battstat->pixgc, i + 2,
-                       pixel_offset_bottom[i] - 1, i + 2,
-                       pixel_offset_bottom[i] - progress_value);
+        pixbuf_draw_line (pixbuf, &color[i], i + 2,
+                          pixel_offset_bottom[i] - 1, i + 2,
+                          pixel_offset_bottom[i] - progress_value);
     }
 
     for( i = 0; i < G_N_ELEMENTS( orange ); i++ )
@@ -815,16 +800,14 @@ update_battery_image (ProgressData *battstat, int batt_percent, int batt_time)
 
       if (progress_value < 33)
       {
-        gdk_gc_set_foreground (battstat->pixgc, &darkcolor[i]);
-		     
         if (battstat->horizont)
-          gdk_draw_line (pixmap, battstat->pixgc,
-                         pixel_offset_bottom[i] - progress_value - 1,
-                         i + 2, x, i + 2);
+          pixbuf_draw_line (pixbuf, &darkcolor[i],
+                            pixel_offset_bottom[i] - progress_value - 1,
+                            i + 2, x, i + 2);
         else
-          gdk_draw_line (pixmap, battstat->pixgc, i + 2,
-                         pixel_offset_bottom[i] - progress_value - 1,
-                         i + 2, x);
+          pixbuf_draw_line (pixbuf, &darkcolor[i], i + 2,
+                            pixel_offset_bottom[i] - progress_value - 1,
+                            i + 2, x);
       }
     }
   }
@@ -832,15 +815,14 @@ update_battery_image (ProgressData *battstat, int batt_percent, int batt_time)
   /* Store our newly created pixmap into the GtkImage.  This results in
      the last reference to the old pixmap/mask being dropped.
   */
-  gtk_image_set_from_pixmap( GTK_IMAGE(battstat->battery),
-                             pixmap, pixmask );
+  gtk_image_set_from_pixbuf( GTK_IMAGE(battstat->battery),
+                             pixbuf);
 
   /* The GtkImage does not assume a reference to the pixmap or mask;
      you still need to unref them if you own references. GtkImage will
      add its own reference rather than adopting yours.
   */
-  g_object_unref( G_OBJECT(pixmap) );
-  g_object_unref( G_OBJECT(pixmask) );
+  g_object_unref( G_OBJECT(pixbuf) );
 }
 
 /* Update the text label that either shows the percentage of time left.
@@ -871,23 +853,6 @@ update_percent_label( ProgressData *battstat, BatteryStatus *info )
   g_free (new_label);
 }
 
-/* Utility function to create a copy of a GdkPixmap */
-static GdkPixmap *
-copy_gdk_pixmap( GdkPixmap *src, GdkGC *gc )
-{
-  gint height, width;
-  GdkPixmap *dest;
-
-  gdk_drawable_get_size( GDK_DRAWABLE( src ), &width, &height );
-
-  dest = gdk_pixmap_new( GDK_DRAWABLE( src ), width, height, -1 );
-
-  gdk_draw_drawable( GDK_DRAWABLE( dest ), gc, GDK_DRAWABLE( src ),
-                     0, 0, 0, 0, width, height );
-
-  return dest;
-}
-
 /* Determine what status icon we ought to be displaying and change the
    status icon to display it if it is different from what we are currently
    showing.
@@ -933,26 +898,14 @@ possibly_update_status_icon( ProgressData *battstat, BatteryStatus *info )
        battstat->last_pixmap_index != STATUS_PIXMAP_METER) )
   {
     GdkColor *colour;
-    GdkPixmap *meter;
+    GdkPixbuf *meter;
     guint progress_value;
     gint i, x;
 
-    /* We keep this pixgc allocated so we don't have to alloc/free it every
-       time.  A widget has to be realized before it has a valid ->window so
-       we do that here for battstat->applet just in case it's not already done.
-    */
-    if( battstat->pixgc == NULL )
-    {
-      gtk_widget_realize( battstat->applet );
-      battstat->pixgc = gdk_gc_new( gtk_widget_get_window (battstat->applet) );
-    }
-
     /* Pull in a clean version of the icons so that we don't paint over
-       top of the same icon over and over.  We neglect to free/update the
-       statusmask here since it will always stay the same.
+       top of the same icon over and over.
      */
-    meter = copy_gdk_pixmap( statusimage[STATUS_PIXMAP_METER],
-                             battstat->pixgc );
+    meter = gdk_pixbuf_copy ( statusimage[STATUS_PIXMAP_METER]);
 
     if (batt_life <= battstat->red_val)
     {
@@ -975,21 +928,19 @@ possibly_update_status_icon( ProgressData *battstat, BatteryStatus *info )
     
     for( i = 0; i < 10; i++ )
     {
-      gdk_gc_set_foreground( battstat->pixgc, &colour[(i * 13 / 10)] );
-
       if( i >= 2 && i <= 7 )
 	x = 17;
       else
 	x = 16;
 
-      gdk_draw_line( meter, battstat->pixgc,
-		     i + 1, x - progress_value,
-		     i + 1, x );
+      pixbuf_draw_line( meter, &colour[(i * 13 / 10)],
+                        i + 1, x - progress_value,
+                        i + 1, x );
     }
 
     /* force a redraw immediately */
-    gtk_image_set_from_pixmap( GTK_IMAGE (battstat->status),
-                               meter, statusmask[STATUS_PIXMAP_METER] );
+    gtk_image_set_from_pixbuf( GTK_IMAGE (battstat->status),
+                               meter);
 
     /* free our private pixmap copy */
     g_object_unref( G_OBJECT( meter ) );
@@ -997,9 +948,8 @@ possibly_update_status_icon( ProgressData *battstat, BatteryStatus *info )
   }
   else if( pixmap_index != battstat->last_pixmap_index )
   {
-    gtk_image_set_from_pixmap (GTK_IMAGE (battstat->status),
-                               statusimage[pixmap_index],
-                               statusmask[pixmap_index]);
+    gtk_image_set_from_pixbuf (GTK_IMAGE (battstat->status),
+                               statusimage[pixmap_index]);
     battstat->last_pixmap_index = pixmap_index;
   }
 }
@@ -1163,9 +1113,6 @@ destroy_applet( GtkWidget *widget, ProgressData *battstat )
   if (battstat->timeout_id)
     g_source_remove (battstat->timeout_id);
 
-  if( battstat->pixgc )
-    g_object_unref( G_OBJECT(battstat->pixgc) );
-
   g_object_unref( G_OBJECT(battstat->status) );
   g_object_unref( G_OBJECT(battstat->percent) );
   g_object_unref( G_OBJECT(battstat->battery) );
@@ -1573,9 +1520,9 @@ create_layout(ProgressData *battstat)
   g_object_ref( battstat->status );
   g_object_ref( battstat->percent );
   g_object_ref( battstat->battery );
-  g_object_ref_sink( GTK_OBJECT( battstat->status ) );
-  g_object_ref_sink( GTK_OBJECT( battstat->percent ) );
-  g_object_ref_sink( GTK_OBJECT( battstat->battery ) );
+  g_object_ref_sink( G_OBJECT( battstat->status ) );
+  g_object_ref_sink( G_OBJECT( battstat->percent ) );
+  g_object_ref_sink( G_OBJECT( battstat->battery ) );
 
   /* Let reconfigure_layout know that the table is currently empty. */
   battstat->layout.status = LAYOUT_NONE;
@@ -1640,7 +1587,6 @@ battstat_applet_fill (PanelApplet *applet)
   battstat->horizont = TRUE;
   battstat->battery_low_dialog = NULL;
   battstat->battery_low_label = NULL;
-  battstat->pixgc = NULL;
   battstat->timeout = -1;
   battstat->timeout_id = 0;
 
@@ -1702,7 +1648,6 @@ battstat_applet_factory (PanelApplet *applet,
 
 PANEL_APPLET_OUT_PROCESS_FACTORY ("BattstatAppletFactory",
 				  PANEL_TYPE_APPLET,
-				  "battstat",
 				  battstat_applet_factory,
 				  NULL)
       
