@@ -47,72 +47,54 @@ shift_right(LoadGraph *g)
 static void
 load_graph_draw (LoadGraph *g)
 {
+    GtkStyle *style;
     guint i, j;
-	
+    cairo_t *cr;
+
     /* we might get called before the configure event so that
      * g->disp->allocation may not have the correct size
      * (after the user resized the applet in the prop dialog). */
 
-    if (!g->pixmap)
-		g->pixmap = gdk_pixmap_new (g->disp->window,
-					    g->draw_width, g->draw_height,
-					    gtk_widget_get_visual (g->disp)->depth);
+    if (!g->surface)
+		g->surface = gdk_window_create_similar_surface (gtk_widget_get_window (g->disp),
+					                        CAIRO_CONTENT_COLOR,
+                                                                g->draw_width, g->draw_height);
 	
-    /* Create GC if necessary. */
-    if (!g->gc)
-    {
-		g->gc = gdk_gc_new (g->disp->window);
-		gdk_gc_copy (g->gc, g->disp->style->black_gc);
-    }
+    style = gtk_widget_get_style (g->disp);
 
-    /* Allocate colors. */
-    if (!g->colors_allocated)
-    {
-		GdkColormap *colormap;
-
-		colormap = gdk_drawable_get_colormap (g->disp->window);
-		
-		for (i = 0; i < g->n; i++)
-	 	   gdk_colormap_alloc_color (colormap, &(g->colors [i]),
-					     FALSE, TRUE);
-
-		g->colors_allocated = 1;
-    }
-	
-    /* Erase Rectangle */
-    gdk_draw_rectangle (g->pixmap,
-			g->disp->style->black_gc,
-			TRUE, 0, 0,
-			g->draw_width,
-			g->draw_height);
+    cr = cairo_create (g->surface);
+    cairo_set_line_width (cr, 1.0);
+    cairo_set_line_cap (cr, CAIRO_LINE_CAP_SQUARE);
 
     for (i = 0; i < g->draw_width; i++)
 		g->pos [i] = g->draw_height - 1;
 
     for (j = 0; j < g->n; j++)
     {
-		gdk_gc_set_foreground (g->gc, &(g->colors [j]));
+		gdk_cairo_set_source_color (cr, &(g->colors [j]));
 
 		for (i = 0; i < g->draw_width; i++) {
 			if (g->data [i][j] != 0) {
-				gdk_draw_line (g->pixmap, g->gc,
-					       g->draw_width - i - 1,
-					       g->pos[i],
-					       g->draw_width - i - 1,
-					       g->pos[i] - (g->data [i][j] - 1));
+                                cairo_move_to (cr,
+                                               g->draw_width - i - 0.5,
+                                               g->pos[i] + 0.5);
+                                cairo_line_to (cr,
+                                               g->draw_width - i - 0.5,
+					       g->pos[i] - (g->data [i][j] - 0.5));
 
 				g->pos [i] -= g->data [i][j];
 			}
 		}
+
+                cairo_stroke (cr);
     }
 	
-    gdk_draw_drawable (g->disp->window,
-		       g->disp->style->fg_gc [GTK_WIDGET_STATE(g->disp)],
-		       g->pixmap,
-		       0, 0,
-		       0, 0,
-		       g->draw_width,
-		       g->draw_height);
+    cairo_destroy (cr);
+
+    cr = gdk_cairo_create (gtk_widget_get_window (g->disp));
+    cairo_set_source_surface (cr, g->surface, 0, 0);
+    cairo_paint (cr);
+    cairo_destroy (cr);
 }
 
 /* Updates the load graph when the timeout expires */
@@ -126,8 +108,8 @@ load_graph_update (LoadGraph *g)
 
     if (g->tooltip_update)
 	multiload_applet_tooltip_update(g);
-    else		
-	g->get_data (g->draw_height, g->data [0], g);
+
+    g->get_data (g->draw_height, g->data [0], g);
 
     load_graph_draw (g);
     return TRUE;
@@ -155,14 +137,9 @@ load_graph_unalloc (LoadGraph *g)
     g->size = panel_applet_gconf_get_int(g->multiload->applet, "size", NULL);
     g->size = MAX (g->size, 10);
 
-    if (g->pixmap) {
-		g_object_unref (g->pixmap);
-		g->pixmap = NULL;
-    }
-
-    if (g->gc) {
-		g_object_unref (g->gc);
-		g->gc = NULL;
+    if (g->surface) {
+		cairo_surface_destroy (g->surface);
+		g->surface = NULL;
     }
 
     g->allocated = FALSE;
@@ -192,35 +169,26 @@ static gint
 load_graph_configure (GtkWidget *widget, GdkEventConfigure *event,
 		      gpointer data_ptr)
 {
+    GtkAllocation allocation;
     LoadGraph *c = (LoadGraph *) data_ptr;
     
     load_graph_unalloc (c);
 
-    c->draw_width = c->disp->allocation.width;
-    c->draw_height = c->disp->allocation.height;
+    gtk_widget_get_allocation (c->disp, &allocation);
+
+    c->draw_width = allocation.width;
+    c->draw_height = allocation.height;
     c->draw_width = MAX (c->draw_width, 1);
     c->draw_height = MAX (c->draw_height, 1);
     
     load_graph_alloc (c);
  
-    if (!c->pixmap)
-	c->pixmap = gdk_pixmap_new (c->disp->window,
-				    c->draw_width,
-				    c->draw_height,
-				    gtk_widget_get_visual (c->disp)->depth);
+    if (!c->surface)
+	c->surface = gdk_window_create_similar_surface (gtk_widget_get_window (c->disp),
+                                                        CAIRO_CONTENT_COLOR,
+				                        c->draw_width, c->draw_height);
 
-    gdk_draw_rectangle (c->pixmap,
-			widget->style->black_gc,
-			TRUE, 0,0,
-			c->draw_width,
-			c->draw_height);
-    gdk_draw_drawable (widget->window,
-		       c->disp->style->fg_gc [GTK_WIDGET_STATE(widget)],
-		       c->pixmap,
-		       0, 0,
-		       0, 0,
-		       c->draw_width,
-		       c->draw_height);
+    gtk_widget_queue_draw (widget);
 
     return TRUE;
 }
@@ -230,13 +198,14 @@ load_graph_expose (GtkWidget *widget, GdkEventExpose *event,
 		   gpointer data_ptr)
 {
     LoadGraph *g = (LoadGraph *) data_ptr;
+    cairo_t *cr;
 
-    gdk_draw_drawable (widget->window,
-		       widget->style->fg_gc [GTK_WIDGET_STATE(widget)],
-		       g->pixmap,
-		       event->area.x, event->area.y,
-		       event->area.x, event->area.y,
-		       event->area.width, event->area.height);
+    cr = gdk_cairo_create (event->window);
+
+    cairo_set_source_surface (cr, g->surface, 0, 0);
+    cairo_paint (cr);
+
+    cairo_destroy (cr);
 
     return FALSE;
 }
@@ -247,7 +216,9 @@ load_graph_destroy (GtkWidget *widget, gpointer data_ptr)
     LoadGraph *g = (LoadGraph *) data_ptr;
 
     load_graph_stop (g);
- 
+    netspeed_delete(g->netspeed_in);
+    netspeed_delete(g->netspeed_out);
+
     gtk_widget_destroy(widget);
 }
 
@@ -259,6 +230,29 @@ load_graph_clicked (GtkWidget *widget, GdkEventButton *event, LoadGraph *load)
 	return FALSE;
 }
 
+static gboolean
+load_graph_enter_cb(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
+{
+	LoadGraph *graph;
+	graph = (LoadGraph *)data;
+
+	graph->tooltip_update = TRUE;
+	multiload_applet_tooltip_update(graph);
+
+	return TRUE;
+}
+
+static gboolean
+load_graph_leave_cb(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
+{
+	LoadGraph *graph;
+	graph = (LoadGraph *)data;
+
+	graph->tooltip_update = FALSE;
+
+	return TRUE;
+}
+
 static void
 load_graph_load_config (LoadGraph *g)
 {
@@ -289,7 +283,8 @@ load_graph_new (MultiloadApplet *ma, guint n, const gchar *label,
     PanelAppletOrient orient;
     
     g = g_new0 (LoadGraph, 1);
-
+    g->netspeed_in = netspeed_new(g);
+    g->netspeed_out = netspeed_new(g);
     g->visible = visible;
     g->name = name;
     g->n = n;
@@ -301,9 +296,9 @@ load_graph_new (MultiloadApplet *ma, guint n, const gchar *label,
     g->show_frame = TRUE;
     g->multiload = ma;
 		
-    g->main_widget = gtk_vbox_new (FALSE, FALSE);
+    g->main_widget = gtk_vbox_new (FALSE, 0);
 
-    g->box = gtk_vbox_new (FALSE, FALSE);
+    g->box = gtk_vbox_new (FALSE, 0);
     
     orient = panel_applet_get_orient (g->multiload->applet);
     switch (orient)
@@ -360,8 +355,12 @@ load_graph_new (MultiloadApplet *ma, guint n, const gchar *label,
 			G_CALLBACK (load_graph_configure), g);
     g_signal_connect (G_OBJECT(g->disp), "destroy",
 			G_CALLBACK (load_graph_destroy), g);
-    g_signal_connect (g->disp, "button-press-event",
+    g_signal_connect (G_OBJECT(g->disp), "button-press-event",
 		        G_CALLBACK (load_graph_clicked), g);
+    g_signal_connect (G_OBJECT(g->disp), "enter-notify-event",
+                      G_CALLBACK(load_graph_enter_cb), g);
+    g_signal_connect (G_OBJECT(g->disp), "leave-notify-event",
+                      G_CALLBACK(load_graph_leave_cb), g);
 	
     gtk_box_pack_start (GTK_BOX (g->box), g->disp, TRUE, TRUE, 0);    
     gtk_widget_show_all(g->box);
