@@ -110,10 +110,10 @@ applet_key_cb (GtkWidget         *widget,
 {
 	switch (event->keyval)
 	{
-		case GDK_KP_Space:
-		case GDK_space:
-		case GDK_KP_Enter:
-		case GDK_Return:
+		case GDK_KEY_KP_Space:
+		case GDK_KEY_space:
+		case GDK_KEY_KP_Enter:
+		case GDK_KEY_Return:
 			stickynote_show_notes (TRUE);
 			return TRUE;
 	}
@@ -133,8 +133,6 @@ gboolean applet_cross_cb(GtkWidget *widget, GdkEventCrossing *event, StickyNotes
 /* Applet Callback : On focus (in or out) of the applet. */
 gboolean applet_focus_cb(GtkWidget *widget, GdkEventFocus *event, StickyNotesApplet *applet)
 {
-	applet->prelighted = event->in;
-
 	stickynotes_applet_update_icon(applet);
 
 	return FALSE;
@@ -165,8 +163,10 @@ void install_check_click_on_desktop (void)
 		return;
 	}
 
-	/* Access the desktop window */
-	window = gdk_window_foreign_new (desktop_window);
+	/* Access the desktop window. desktop_window is the root window for the
+         * default screen, so we know using gdk_display_get_default() is correct. */
+	window = gdk_x11_window_foreign_new_for_display (gdk_display_get_default (),
+	                                                 desktop_window);
 
 	/* It may contain an atom to tell us which other window to monitor */
 	user_time_window = gdk_x11_get_xatom_by_name ("_NET_WM_USER_TIME_WINDOW");
@@ -175,26 +175,27 @@ void install_check_click_on_desktop (void)
 	{
 		/* Looks like the atoms are there */
 		Atom actual_type;
-		int  actual_format;
-		long nitems;
-		long bytes;
+		int actual_format;
+		gulong nitems;
+		gulong bytes;
 		Window *data;
 
 		/* We only use this extra property if the actual user-time property's missing */
-		int  status = XGetWindowProperty( GDK_DISPLAY(), desktop_window, user_time,
+		int  status = XGetWindowProperty( GDK_DISPLAY_XDISPLAY(gdk_window_get_display(window)), desktop_window, user_time,
 					0, 4, False, AnyPropertyType, &actual_type, &actual_format,
 					&nitems, &bytes, (unsigned char **)&data );
 		if (actual_type == None)
 		{
 			/* No user-time property, so look for the user-time-window */
-			status = XGetWindowProperty( GDK_DISPLAY(), desktop_window, user_time_window,
+                        status = XGetWindowProperty( GDK_DISPLAY_XDISPLAY(gdk_window_get_display(window)), desktop_window, user_time_window,
 					0, 4, False, AnyPropertyType, &actual_type, &actual_format,
 					&nitems, &bytes, (unsigned char **)&data );
 			if (actual_type != None)
 			{
 				/* We have another window to monitor */
 				desktop_window = *data;
-				window = gdk_window_foreign_new (desktop_window);
+				window = gdk_x11_window_foreign_new_for_display (gdk_window_get_display (window),
+				                                                 desktop_window);
 			}
 		}
 	}
@@ -229,49 +230,6 @@ void applet_size_allocate_cb(GtkWidget *widget, GtkAllocation *allocation, Stick
 	return;
 }
 
-/* Applet Callback : Change the applet background. */
-void
-applet_change_bg_cb (PanelApplet *panel_applet,
-		     PanelAppletBackgroundType type,
-		     GdkColor *color,
-		     GdkPixmap *pixmap,
-		     StickyNotesApplet *applet)
-{
-	/* Taken from TrashApplet */
-	GtkRcStyle *rc_style;
-	GtkStyle *style;
-
-	if (!applet) g_print ("arrg, no applet!\n");
-
-	/* reset style */
-	gtk_widget_set_style (GTK_WIDGET (applet->w_applet), NULL);
-	rc_style = gtk_rc_style_new ();
-	gtk_widget_modify_style (GTK_WIDGET (applet->w_applet), rc_style);
-	g_object_unref (rc_style);
-
-	switch (type)
-	{
-		case PANEL_NO_BACKGROUND:
-			break;
-		case PANEL_COLOR_BACKGROUND:
-			gtk_widget_modify_bg (GTK_WIDGET (applet->w_applet),
-					GTK_STATE_NORMAL, color);
-			break;
-		case PANEL_PIXMAP_BACKGROUND:
-			style = gtk_style_copy (
-					gtk_widget_get_style (GTK_WIDGET (applet->w_applet)));
-			if (style->bg_pixmap[GTK_STATE_NORMAL])
-				g_object_unref (
-					style->bg_pixmap[GTK_STATE_NORMAL]);
-			style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (
-					pixmap);
-			gtk_widget_set_style (
-					GTK_WIDGET (applet->w_applet), style);
-			g_object_unref (style);
-			break;
-	}
-}
-
 /* Applet Callback : Deletes the applet. */
 void applet_destroy_cb (PanelApplet *panel_applet, StickyNotesApplet *applet)
 {
